// ConsoleApplication338.cpp:      .
//

#include"stdafx.h"
#include <iostream>
#include <clocale>
#include <list>
#include <cstdlib>
using namespace std;

enum Type { ARR, MAX_ARR };//  :      
enum Field { FREE, ATACK, CASTLE, KNIGHT, BISHOP, QUEEN, KING };


class ChessBoard
{
private:
	int x;                  //  
	int y;                  //  
	Field **mass;           // 

	void createBoard();     //    
	void clearBoard();      //     	
	void deleteBoard();     //   -  

							//  ,        

	bool recursiveArrangement(list<ChessBoard> &lst, ChessBoard board, Field field, int k, int n);

	//  ,       
	bool atackFigure(int x, int y, Field field);

public:
	//  -   
	ChessBoard(int x = 8, int y = 8);
	//  
	ChessBoard(const ChessBoard &cpy);

	//   =
	ChessBoard & operator= (const ChessBoard &cpy);

	//    
	void printBoard() const;

	//      
	//     -      false
	bool setFigure(int x, int y, Field field);

	//       
	// n -      
	bool setNextFigureAfterLast(Field Field, int n = 0);

	//       
	//     lst
	//   
	//      - n
	int arrangement(list<ChessBoard> &lst, Field field, int n = 1);

	//    
	int maxArrangement(Field field);

	// 
	~ChessBoard();
};

ostream & operator<< (ostream &os, const ChessBoard &ob)
{
	ob.printBoard();
	return os;
}

void ChessBoard::createBoard()
{
	mass = new Field*[x];
	for (int i = 0; i < x; i++)
	{
		mass[i] = new Field[y];
	}

	clearBoard();
}

void ChessBoard::clearBoard()
{
	for (int i = 0; i < x; i++)
		for (int j = 0; j < y; j++)
			mass[i][j] = FREE;
}

void ChessBoard::deleteBoard()
{
	for (int i = 0; i < x; i++)
	{
		delete[]mass[i];
	}
	delete[]mass;
}

bool ChessBoard::recursiveArrangement(std::list<ChessBoard> &lst, ChessBoard board, Field field, int k, int n)
{
	if (n == 0)
	{
		lst.push_back(board);
		//    
		return false;
	}

	if (board.setNextFigureAfterLast(field, k) == false)
	{
		return false;
	}

	int i = 0;
	while (recursiveArrangement(lst, board, field, i++, n - 1))
		continue;

	return true;
}

bool ChessBoard::atackFigure(int x, int y, Field field)
{
	if (x >= this->x || y >= this->y) //   ;
		return false;

	switch (field)
	{
	case CASTLE:
		for (int i = 0; i < this->x; i++)
		{
			if (mass[i][y] == FREE)
				mass[i][y] = ATACK;
		}
		for (int j = 0; j < this->y; j++)
		{
			if (mass[x][j] == FREE)
				mass[x][j] = ATACK;
		}
		mass[x][y] = CASTLE;
		break;
	case KNIGHT:
		mass[x][y] = KNIGHT;
		if (x + 2 < this->x && y + 1 < this->y)
			mass[x + 2][y + 1] = ATACK;
		if (x + 2 < this->x && y - 1 >= 0)
			mass[x + 2][y - 1] = ATACK;
		if (x + 1 < this->x && y + 2 < this->y)
			mass[x + 1][y + 2] = ATACK;
		if (x + 1 < this->x && y - 2 >= 0)
			mass[x + 1][y - 2] = ATACK;
		if (x - 1 >= 0 && y + 2 < this->y)
			mass[x - 1][y + 2] = ATACK;
		if (x - 1 >= 0 && y - 2 >= 0)
			mass[x - 1][y - 2] = ATACK;
		if (x - 2 >= 0 && y + 1 < this->y)
			mass[x - 2][y + 1] = ATACK;
		if (x - 2 >= 0 && y - 1 >= 0)
			mass[x - 2][y - 1] = ATACK;
		break;
	case BISHOP:
		for (int i = 1; i < this->x || i < this->y; i++)
		{
			if (x + i < this->x && y + i < this->y)
				mass[x + i][y + i] = ATACK;
			if (x + i < this->x && y - i >= 0)
				mass[x + i][y - i] = ATACK;
			if (x - i >= 0 && y + i < this->y)
				mass[x - i][y + i] = ATACK;
			if (x - i >= 0 && y - i >= 0)
				mass[x - i][y - i] = ATACK;
		}
		mass[x][y] = BISHOP;
		break;
	case QUEEN:
		for (int i = 0; i < this->x; i++)
		{
			mass[i][y] = ATACK;
		}
		for (int j = 0; j < this->y; j++)
		{
			mass[x][j] = ATACK;
		}
		for (int i = 1; i < this->x || i < this->y; i++)
		{
			if (x + i < this->x && y + i < this->y)
				mass[x + i][y + i] = ATACK;
			if (x + i < this->x && y - i >= 0)
				mass[x + i][y - i] = ATACK;
			if (x - i >= 0 && y + i < this->y)
				mass[x - i][y + i] = ATACK;
			if (x - i >= 0 && y - i >= 0)
				mass[x - i][y - i] = ATACK;
		}
		mass[x][y] = QUEEN;
		break;
	case KING:
		mass[x][y] = KING;
		if (x + 1 < this->x)
			mass[x + 1][y] = ATACK;
		if (x + 1 < this->x && y - 1 >= 0)
			mass[x + 1][y - 1] = ATACK;
		if (y + 1 < this->y)
			mass[x][y + 1] = ATACK;
		if (x + 1 < this->x && y + 1 < this->y)
			mass[x + 1][y + 1] = ATACK;
		if (x - 1 >= 0 && y + 1 < this->y)
			mass[x - 1][y + 1] = ATACK;
		if (y - 1 >= 0)
			mass[x][y - 1] = ATACK;
		if (x - 1 >= 0 && y - 1 >= 0)
			mass[x - 1][y - 1] = ATACK;
		if (x - 1 >= 0)
			mass[x - 1][y] = ATACK;


		break;
	default:
		return false;
	}
	return true;
}

// 
ChessBoard::ChessBoard(int x, int y)
{
	this->x = x;
	this->y = y;

	createBoard();
}

//  
ChessBoard::ChessBoard(const ChessBoard &cpy)
{
	x = cpy.x;
	y = cpy.y;

	createBoard();

	for (int i = 0; i < x; i++)
		for (int j = 0; j < y; j++)
			mass[i][j] = cpy.mass[i][j];
}

ChessBoard & ChessBoard::operator=(const ChessBoard &cpy)
{
	x = cpy.x;
	y = cpy.y;

	deleteBoard();
	createBoard();

	for (int i = 0; i < x; i++)
		for (int j = 0; j < y; j++)
			mass[i][j] = cpy.mass[i][j];

	return *this;
}

void ChessBoard::printBoard() const
{
	for (int i = 0; i < x; i++)
	{
		for (int j = 0; j < y; j++)
		{
			switch (mass[i][j])
			{
			case FREE:
				cout << ". ";
				break;
			case ATACK:
				cout << "* ";
				break;
			case CASTLE:
				cout << "C ";
				break;
			case KNIGHT:
				cout << "K ";
				break;
			case BISHOP:
				cout << "B ";
				break;
			case QUEEN:
				cout << "Q ";
				break;
			case KING:
				cout << "KG";
				break;
			}
		}
		cout << '\n';
	}
}

bool ChessBoard::setFigure(int x, int y, Field field)
{
	//      
	if (x >= this->x || x < 0 || y >= this->y || y < 0)
		return false;

	//     -   
	if (mass[x][y] != FREE)
		return false;

	atackFigure(x, y, field);
	return true;
}

bool ChessBoard::setNextFigureAfterLast(Field field, int n)
{
	int a = -1;
	int b = -1;
	int counter = n;
	for (int i = 0; i < x; i++)
	{
		for (int j = 0; j < y; j++)
		{
			if (mass[i][j] != FREE && mass[i][j] != ATACK)
			{
				a = -1;
				b = -1;
				counter = n;
			}

			if (mass[i][j] == FREE)
			{
				if (counter-- == 0)
				{
					a = i;
					b = j;
				}
			}
		}
	}

	if (setFigure(a, b, field))
		return true;
	else
		return false;       //      
}

int ChessBoard::arrangement(list<ChessBoard> &lst, Field field, int n)
{
	int i = 0;
		while (recursiveArrangement(lst, *this, field, i++, n))
			continue;
       return lst.size();
}

//    
int ChessBoard::maxArrangement(Field field)
{
	int max = 2;
	while (true)
	{
		cout << "max = " << max << endl;
		int i = 0;
		list<ChessBoard> lst;
		while (recursiveArrangement(lst, *this, field, i++, max))
			continue;

		if (lst.size() == 0)
			return max - 1;

		max++;
	}
}

ChessBoard::~ChessBoard()
{
	deleteBoard();
}


int main()
{

	// 
	setlocale(LC_ALL, "Russian");


	int line;                       //  
	int column;                     //  
	Field figure;                   //  
	int n;                          //  

	Type a;   // 


			  //  
	for (;;)
	{
		//  
		system("cls");

		//   
		std::cout << " : ";
		cin >> line;
		if (line == false)
		{
			cin.clear();
			while (cin.get() != '\n')
				continue;

			cout << " \n";
			system("pause");
			continue;
		}

		if (line <= 0)
		{
			cout << "     0\n";
			system("pause");
			continue;
		}
		//    

		//   
		cout << " : ";
		cin >> column;
		if (column == false)
		{
			cin.clear();
			while (cin.get() != '\n')
				continue;

			cout << " \n";
			system("pause");
			continue;
		}
		if (column <= 0)
		{
			cout << "     0\n";
			system("pause");
			continue;
		}
		//    

		//   
		cout << " :\n"
			<< "1 - \n"
			<< "2 - \n"
			<< "3 - \n"
			<< "4 - \n"
			<< "5- \n";
		int answer;
		cin >> answer;
		if (answer == false)
		{
			cin.clear();
			while (cin.get() != '\n')
				continue;

			cout << " \n";
			system("pause");
			continue;
		}
		switch (answer)
		{
		case 1: figure = CASTLE;    break;
		case 2: figure = KNIGHT;    break;
		case 3: figure = BISHOP;    break;
		case 4: figure = QUEEN;     break;
		case 5: figure = KING;      break;
		default:
			cout << "  \n";
			system("pause");
			continue;
		}
		//    

		//   
		cout << " \n"
			<< "1 - \n"
			<< "2 -   \n";

		cin >> answer;
		switch (answer)
		{
		case 1: a = ARR;        break;
		case 2: a = MAX_ARR;    break;
		default:
			cout << "Error\n";
			system("pause");
			continue;
		}
		//    

		if (a == ARR)
		{
			//   
			cout << " : ";
			cin >> n;
			if (n == false)
			{
				cin.clear();
				while (cin.get() != '\n')
					continue;

				cout << " \n";
				system("pause");
				continue;
			}
			if (n <= 0)
			{
				cout << "     0\n";
				system("pause");
				continue;
			}
			//    
		}

		break;
	}

	if (a == ARR)
	{
		list<ChessBoard> lst;
		try
		{
			ChessBoard board(line, column);
			board.arrangement(lst, figure, n);
		}
		catch (bad_alloc &ex)//  
		{
			cerr << ex.what() << endl;
		}

		for (list<ChessBoard>::iterator it = lst.begin(); it != lst.end(); it++)
			cout << *it << '\n';

		cout << " " << lst.size();
		switch (lst.size() % 10)
		{
		case 0:
		case 5:
		case 6:
		case 7:
		case 8:
		case 9:
			cout << " " << '\n';
			break;

		case 1:
			cout << " " << '\n';
			break;

		case 2:
		case 3:
		case 4:
			cout << " " << '\n';
			break;
		}
		system("pause");
	}

	else if (a == MAX_ARR)
	{
		ChessBoard board(line, column);
		int max = board.maxArrangement(figure);
		cout << max;
	}




	system("pause");
	return 0;
}


